! Fortran-Python hybrid module
! Elias Rabel
module mymath
use forpy_mod
use, intrinsic :: iso_c_binding
use, intrinsic :: iso_fortran_env 
implicit none

public :: square

PRIVATE

! You need to declare exactly one PythonModule and PythonMethodTable
! at Fortran module level
type(PythonModule), save :: mod_def
type(PythonMethodTable), save :: method_table

CONTAINS

! Initialisation function for Python 3
! called when importing module
! must use bind(c, name="PyInit_<module name>")
! return value must be type(c_ptr), use the return value of PythonModule%init
function PyInit_mymath() bind(c, name="PyInit_mymath") result(m)
  type(c_ptr) :: m
  m = init()
end function

! Initialisation function for Python 2
! called when importing module
! must use bind(c, name="init<module name>")
! Initialisation function for Python 2
! called when importing module
! must be called init<module name>
subroutine initmymath() bind(c, name="initmymath")
  type(c_ptr) :: m
  m = init()
end subroutine

function init() result(m)
  type(c_ptr) :: m
  integer :: ierror
  type(object) :: pi
  
  ierror = forpy_initialize()
  
  call method_table%init(1)
  
  ! must add function square to method table to be able to use it in Python
  
  call method_table%add_method("square", &                  ! method name
                               "Computes square of a float", &  !doc-string
                               METH_VARARGS, &                  ! this method takes arguments but no keyword arguments
                               c_funloc(mymath_square_python))  ! address of Fortran function to add
    
  m = mod_def%init("mymath", "A Python extension with a math function", method_table)
end function

! Implementation of our Python method
!
! Corresponding Python method shall allow arguments but no keyword arguments
! -> We need 2 "type(c_ptr), value" arguments
! First arg is c_ptr to module, second is c_ptr to argument tuple
! Return value must be type(c_ptr)
function mymath_square_python(self_ptr, args_ptr) result(r) bind(c)
  ! used bind(c) attribute to ensure that C calling conventions are used
  ! used a long method name to avoid name clashes with other C libs
  ! 
  type(c_ptr), value :: self_ptr
  type(c_ptr), value :: args_ptr
  type(c_ptr) :: r
  
  type(tuple) :: args
  type(object) :: retval
  integer :: ierror
  integer :: num_args
  real(kind=real64) :: a_number

  r = C_NULL_PTR   ! in case of an exception return C_NULL_PTR
 
  ! use unsafe_cast_from_c_ptr to cast from c_ptr to tuple
  call unsafe_cast_from_c_ptr(args, args_ptr)

  ! Check if the arguments are OK
  ierror = args%len(num_args)    ! we should also check ierror, but this example does not do complete error checking for simplicity

  if (num_args /= 1) then
    call raise_exception(TypeError, "square expects exactly 1 argument")
    call args%destroy
    return
  endif

  ierror = args%getitem(a_number, 0)
  
  if (ierror == 0) then
    ierror = cast(retval, square(a_number))
    r = retval%get_c_ptr()
  endif  
  
  call args%destroy
  
end function

! Fortran implementation
elemental function square(a)
  real(kind=real64), intent(in) :: a
  real(kind=real64) :: square

  square = a**2
end function

end module
